"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const amqp = tslib_1.__importStar(require("amqplib"));
class rabbit {
    constructor(url) {
        this.STATUS = 'CONNECTING';
        this.messageStore = [];
        this.connection = null;
        this.connectionPromise = this.connector(url);
    }
    connector(url) {
        let connection;
        return Promise.resolve()
            .then(() => {
            //can add url formatting logic here
            if (url !== 'amqp://localhost') {
                url = 'amqp://localhost';
            }
            return url;
        })
            .then((url) => {
            return amqp.connect(url).then((conn) => {
                console.log('[AMQP] Connected');
                this.STATUS = 'CONNECTED';
                this.connection = conn;
                this.sendMessageQueue();
                conn.on('close', (err) => {
                    this.connection = null;
                    this.STATUS = 'RECONNECTING';
                    console.log('[AMQP] lost connection');
                    setTimeout(() => {
                        console.log('[AMQP] attempting to reconnect');
                        this.connector(url);
                    }, 5000);
                });
            });
        })
            .catch((err) => {
            console.log(`[AMQP] ${err}`);
            this.connectionPromise = null;
            this.connection = null;
            setTimeout(() => {
                console.log('[AMQP] attempting to connect');
                this.STATUS = 'RECONNECTING';
                this.connector(url);
            }, 5000);
        });
    }
    sendMessageQueue() {
        this.messageStore.forEach((msg) => {
            this.pub(msg.Queue, msg.Msg);
        });
    }
    getStatus() {
        return this.STATUS;
    }
    getMessageQueue() {
        return this.messageStore;
    }
    pub(Queue, Msg) {
        // console.log('smartpub')
        if (this.connection !== null) {
            this.connection
                .createChannel()
                .then((ch) => {
                ch.assertQueue(Queue).then((ok) => {
                    ch.sendToQueue(Queue, Buffer.from(Msg));
                    console.log(`[AMQP] PUB ${Queue} : ${Msg}`);
                    return ch.close();
                });
            })
                .catch((err) => {
                console.log(`[AMQP] connection - ${err}`);
                this.messageStore.push({ Queue: Queue, Msg: Msg });
            });
        }
        else {
            console.log(`[AMQP] NO CONNECTION`);
            this.messageStore.push({ Queue: Queue, Msg: Msg });
        }
    }
    sub(Queue, callback) {
        if (this.connection !== null) {
            this.connection
                .createChannel()
                .then((ch) => {
                ch.assertQueue(Queue).then((ok) => {
                    ch.consume(Queue, (msg) => {
                        if (msg !== null) {
                            callback(msg.content.toString());
                            ch.ack(msg);
                        }
                        else {
                            console.log(`[AMQP] MSG EMPTY`);
                        }
                    });
                });
            })
                .catch((err) => {
                console.log(`[AMQP] connection - ${err}`);
            });
        }
        else {
            console.log(`[AMQP] NO CONNECTION`);
        }
    }
    createQueue(queue) {
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.assertQueue(queue)
                    .then((ok) => {
                    console.log(`[AMQP] CREATED QUEUE : ${queue}`);
                })
                    .catch((err) => {
                    console.log(`[AMQP] ${err}`);
                });
            });
        }
        else {
            console.log(`[AMQP] NO CONNECTION`);
        }
    }
    deleteQueue(queue) {
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.deleteQueue(queue)
                    .then((ok) => {
                    console.log(`[AMQP] Deleted QUEUE : ${queue}`);
                })
                    .catch((err) => {
                    console.log(`[AMQP] ${err}`);
                });
            });
        }
        else {
            console.log(`[AMQP] NO CONNECTION`);
        }
    }
    createExchange(name, type) {
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.assertExchange(name, type, { durable: false })
                    .then((ok) => {
                    console.log(`[AMQP] CREATED EXCHANGE : ${name}`);
                })
                    .catch((err) => {
                    console.log(`[AMQP] ${err}`);
                });
            });
        }
        else {
            console.log(`[AMQP] NO CONNECTION`);
        }
    }
    deleteExchange(name) {
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.deleteExchange(name)
                    .then((ok) => {
                    console.log(`[AMQP] DELETED EXCHANGE : ${name}`);
                })
                    .catch((err) => {
                    console.log(`[AMQP] ${err}`);
                });
            });
        }
        else {
            console.log(`[AMQP] NO CONNECTION`);
        }
    }
    bindQueue(queueName, exchangeName, pattern, args) {
        if (this.connection !== null) {
            this.connection.createChannel().then((ch) => {
                ch.bindQueue(queueName, exchangeName, '', args)
                    .then((ok) => {
                    console.log(`[AMQP] BINDED QUEUE ${queueName} TO EXCHANGE ${exchangeName}`);
                })
                    .catch((err) => {
                    console.log(`[AMQP] ${err}`);
                });
            });
        }
        else {
            console.log(`[AMQP] NO CONNECTION`);
        }
    }
}
exports.default = rabbit;
